import { EventEmitter, Directive, ElementRef, Input, Output, NgModule } from '@angular/core';
import { use, init } from 'echarts/core';
import { CanvasRenderer } from 'echarts/renderers';
import { interval } from 'rxjs';
import { filter, map } from 'rxjs/operators';

class HtmlHelper {
    static elementIsFocusable(ele) {
        return ele.getAttribute("tabindex") != null;
    }
    static getWidthSensor(element, periodicityInMiliSeconds = 2000) {
        let oldWidth = element.clientWidth;
        return interval(periodicityInMiliSeconds).pipe(filter(ignore => {
            if (Math.abs(oldWidth - element.clientWidth) > 2) {
                oldWidth = element.clientWidth;
                return true;
            }
            return false;
        }), map(ignore => element.clientWidth));
    }
    static getHeightSensor(element, periodicityInMiliSeconds = 2000) {
        let oldHeight = element.clientHeight;
        return interval(periodicityInMiliSeconds).pipe(filter(ignore => {
            if (Math.abs(oldHeight - element.clientHeight) > 2) {
                oldHeight = element.clientHeight;
                return true;
            }
            return false;
        }), map(ignore => element.clientWidth));
    }
}

class EchartsDirective {
    constructor(_el) {
        this._el = _el;
        this.extentions = [];
        this.isResizable = true;
        this.defaultWidth = 400;
        this.defaultHeight = 400;
        this.periodicityInMiliSeconds = 2000;
        this.theme = '';
        this.echartsInstance = new EventEmitter();
    }
    ngOnInit() {
        use([...this.extentions, CanvasRenderer]);
        this._echartsInstance = init(this._el.nativeElement, this.theme, {
            width: this._el.nativeElement.clientWidth === this.defaultWidth ? 400 : undefined,
            height: this._el.nativeElement.clientHeight === 0 ? this.defaultHeight : undefined
        });
        this.echartsInstance.emit(this._echartsInstance);
        this._setParams();
        if (this.isResizable) {
            this._addResizbleFunctionality();
        }
    }
    ngOnChanges(changes) {
        if (changes.options && !changes.options.firstChange) {
            this._setParams();
        }
        if (changes.isResizable && !changes.isResizable.firstChange) {
            if (this.isResizable) {
                this._addResizbleFunctionality();
            }
            else {
                if (this._subscription != null)
                    this._subscription.unsubscribe();
            }
        }
    }
    _addResizbleFunctionality() {
        if (this._subscription != null)
            this._subscription.unsubscribe();
        this._subscription = HtmlHelper.getWidthSensor(this._el.nativeElement).subscribe(() => {
            if (this._echartsInstance != null) {
                this._echartsInstance.resize();
            }
        });
    }
    _setParams() {
        if (this._echartsInstance != null && this.options != null)
            this._echartsInstance.setOption(Object.assign({}, this.options));
    }
    ngOnDestroy() {
        if (this._subscription != null)
            this._subscription.unsubscribe();
    }
}
EchartsDirective.decorators = [
    { type: Directive, args: [{
                selector: "[echarts]"
            },] }
];
EchartsDirective.ctorParameters = () => [
    { type: ElementRef }
];
EchartsDirective.propDecorators = {
    options: [{ type: Input }],
    extentions: [{ type: Input }],
    isResizable: [{ type: Input }],
    defaultWidth: [{ type: Input }],
    defaultHeight: [{ type: Input }],
    periodicityInMiliSeconds: [{ type: Input }],
    theme: [{ type: Input }],
    echartsInstance: [{ type: Output }]
};

class EchartsxModule {
}
EchartsxModule.decorators = [
    { type: NgModule, args: [{
                declarations: [EchartsDirective],
                exports: [EchartsDirective]
            },] }
];

/*
 * Public API Surface of echartsx
 */

/**
 * Generated bundle index. Do not edit.
 */

export { EchartsxModule, EchartsDirective as ɵa };
//# sourceMappingURL=echarts-for-angular.js.map
